@@ -1,5 +1,5 @@
 # test_patch.py -- tests for patch.py
-# Copyright (C) 2010 Jelmer Vernooij <jelmer@samba.org>
+# Copyright (C) 2010 Jelmer Vernooij <jelmer@jelmer.uk>
 #
 # Dulwich is dual-licensed under the Apache License, Version 2.0 and the GNU
 # General Public License as public by the Free Software Foundation; version 2.0
@@ -166,7 +166,7 @@ Date: Thu, 15 Apr 2010 15:40:28 +0200
 Subject:  [Dulwich-users] [PATCH] Added unit tests for
  dulwich.object_store.tree_lookup_path.
 
-From: Jelmer Vernooy <jelmer@debian.org>
+From: Jelmer Vernooij <jelmer@debian.org>
 
 * dulwich/tests/test_object_store.py
   (TreeLookupPathTests): This test case contains a few tests that ensure the
@@ -180,7 +180,7 @@ From: Jelmer Vernooy <jelmer@debian.org>
 1.7.0.4
 """  # noqa: W291
         c, diff, version = git_am_patch_split(BytesIO(text), "utf-8")
-        self.assertEqual(b"Jelmer Vernooy <jelmer@debian.org>", c.author)
+        self.assertEqual(b"Jelmer Vernooij <jelmer@debian.org>", c.author)
         self.assertEqual(b'''\
 Added unit tests for dulwich.object_store.tree_lookup_path.
 
@@ -197,7 +197,7 @@ Date: Thu, 15 Apr 2010 15:40:28 +0200
 Subject:  [Dulwich-users] [PATCH] Added unit tests for
  dulwich.object_store.tree_lookup_path.
 
-From: Jelmer Vernooy <jelmer@debian.org>
+From: Jelmer Vernooij <jelmer@debian.org>
 
 ---
  pixmaps/prey.ico |  Bin 9662 -> 9662 bytes
@@ -281,7 +281,7 @@ class DiffTests(TestCase):
              b'index 0000000..a116b51 644',
              b'--- /dev/null',
              b'+++ b/bar.txt',
-             b'@@ -1,0 +1,2 @@',
+             b'@@ -0,0 +1,2 @@',
              b'+new',
              b'+same'
             ], f.getvalue().splitlines())
@@ -297,7 +297,7 @@ class DiffTests(TestCase):
             b'index a116b51..0000000',
             b'--- a/bar.txt',
             b'+++ /dev/null',
-            b'@@ -1,2 +1,0 @@',
+            b'@@ -1,2 +0,0 @@',
             b'-new',
             b'-same'
             ], f.getvalue().splitlines())
@@ -327,7 +327,7 @@ class DiffTests(TestCase):
             b'index 0000000..76d4bb8 644',
             b'--- /dev/null',
             b'+++ b/added.txt',
-            b'@@ -1,0 +1,1 @@',
+            b'@@ -0,0 +1 @@',
             b'+add',
             b'diff --git a/changed.txt b/changed.txt',
             b'index bf84e48..1be2436 644',
@@ -342,7 +342,7 @@ class DiffTests(TestCase):
             b'index 2c3f0b3..0000000',
             b'--- a/removed.txt',
             b'+++ /dev/null',
-            b'@@ -1,1 +1,0 @@',
+            b'@@ -1 +0,0 @@',
             b'-removed',
             ], f.getvalue().splitlines())
 
@@ -362,7 +362,7 @@ class DiffTests(TestCase):
             b'index 06d0bdd..cc97564 160000',
             b'--- a/asubmodule',
             b'+++ b/asubmodule',
-            b'@@ -1,1 +1,1 @@',
+            b'@@ -1 +1 @@',
             b'-Submodule commit 06d0bdd9e2e20377b3180e4986b14c8549b393e4',
             b'+Submodule commit cc975646af69f279396d4d5e1379ac6af80ee637',
             ], f.getvalue().splitlines())
@@ -399,7 +399,7 @@ class DiffTests(TestCase):
              b'index 0000000..a116b51 644',
              b'--- /dev/null',
              b'+++ b/bar.txt',
-             b'@@ -1,0 +1,2 @@',
+             b'@@ -0,0 +1,2 @@',
              b'+new',
              b'+same'
             ], f.getvalue().splitlines())
@@ -417,7 +417,7 @@ class DiffTests(TestCase):
             b'index a116b51..0000000',
             b'--- a/bar.txt',
             b'+++ /dev/null',
-            b'@@ -1,2 +1,0 @@',
+            b'@@ -1,2 +0,0 @@',
             b'-new',
             b'-same'
             ], f.getvalue().splitlines())
@@ -532,7 +532,7 @@ class DiffTests(TestCase):
             b'index a116b51..06d0bdd 160000',
             b'--- a/bar.txt',
             b'+++ b/bar.txt',
-            b'@@ -1,2 +1,1 @@',
+            b'@@ -1,2 +1 @@',
             b'-new',
             b'-same',
             b'+Submodule commit 06d0bdd9e2e20377b3180e4986b14c8549b393e4',
